@@ -10,8 +10,8 @@ BO_EXCLUDED = set([
     'AutoTurret',
     'PointDefenseDrone',
     'Mothership',
-    'Baneling',
     'Locust',
+    'LocustMP',
     'Changeling',
     'InfestedTerran',
     'Overseer',
@@ -25,6 +25,7 @@ BO_EXCLUDED = set([
     'AutoTurret',
     'InfestedTerran',
     'Interceptor',
+    'SwarmHostMP',
 ])
 
 BUILD_TIMES = {
@@ -33,16 +34,13 @@ BUILD_TIMES = {
     'Marauder': 30,
     'Reaper': 40,
     'Ghost': 40,
+    'BattleHellion': 30,  # Hellbat
     'Hellion': 30,
     'Hellbat': 30,
-    'HellionTank': 30,
     'WidowMine': 40,
     'SiegeTank': 45,
     'Thor': 60,
     'Viking': 42,
-    'VikingFighter': 42,
-    'VikingFlying': 42,
-    'VikingAssault': 42,
     'Medivac': 42,
     'Raven': 60,
     'Banshee': 60,
@@ -67,10 +65,10 @@ BUILD_TIMES = {
     'Drone': 17,
     'Queen': 50,
     'Zergling': 24,
+    'Baneling': 20,
     'Roach': 27,
     'Hydralisk': 33,
     'SwarmHost': 40,
-    'SwarmHostMP': 40,
     'Infestor': 50,
     'Ultralisk': 55,
     'NydusWorm': 20,
@@ -168,7 +166,58 @@ BUILD_TIMES = {
     'Charge': 140,
     'PsiStormTech': 110,
     'PhoenixRangeUpgrade': 90,
+    # unit change buildings
+    'Lair': 80,
+    'Hive': 100,
+    'GreaterSpire': 150,
+    'OrbitalCommand': 35,
+    'PlanetaryFortress': 50,
+    'Overseer': 17,
 }
 
 for key, value in BUILD_TIMES.iteritems():
     BUILD_TIMES[key] *= 16
+
+
+TRACKED_ABILITIES = set([
+    '250mmStrikeCannons',
+    'BuildAutoTurret',
+    'BuildAutoTurret',
+    'CalldownMULE',
+    'Contaminate',
+    'Corruption',
+    'EMPRound',
+    'Feedback',
+    'ForceField',
+    'FungalGrowth',
+    'GravitonBeam',
+    'GuardianShield',
+    'MassRecallMothership',
+    'MothershipMassRecall',
+    'MothershipCorePurifyNexus',
+    'MassRecallMothershipCore',
+    'InfestorNeuralParasite',
+    'BuildPointDefenseDrone',
+    'HallucinationArchon',
+    'HallucinationColossus',
+    'HallucinationHighTemplar',
+    'HallucinationImmortal',
+    'HallucinationPhoenix',
+    'HallucinationProbe',
+    'HallucinationStalker',
+    'HallucinationVoidRay',
+    'HallucinationWarpPrism',
+    'HallucinationZealot',
+    'PsionicStorm',
+    'ScannerSweep',
+    'SeekerMissile',
+    'SniperRound',
+    'SpawnLarva',
+    'ExtraSupplies',
+    'ChronoBoost',
+    'QueenTransfusion',
+    'YamatoGun',
+    'Abduct',
+    'TemporalField',
+    'Envision',
+    ])
