@@ -4,7 +4,7 @@ spawningtool.parser
 """
 import sc2reader
 
-from spawningtool.constants import BO_EXCLUDED, BUILD_TIMES
+from spawningtool.constants import BO_EXCLUDED, BUILD_TIMES, TRACKED_ABILITIES
 from spawningtool.exception import CutoffTimeError, ReplayFormatError, ReadError
 
 
@@ -29,10 +29,16 @@ def _frame_to_time(frame):
 
 
 class TrackerEvent(object):
+    """
+    not necesarily a trackerevent, but event was too generic
+    """
     def __init__(self, frame, supply=None):
         self.frame = frame
         self.supply = supply
 
+    def to_dict(self):
+        raise NotImplementedError
+
     def __unicode__(self):
         if self.supply:
             return '{} {} Event'.format(self.supply, self.frame)
@@ -47,6 +53,15 @@ class BuildEvent(TrackerEvent):
     def is_worker(self):
         return self.name in ['SCV', 'Drone', 'Probe']
 
+    def to_dict(self):
+        return {
+            'frame': self.frame,
+            'time': _frame_to_time(self.frame),
+            'name': self.name,
+            'supply': self.supply,
+            'is_worker': self.is_worker()
+        }
+
     def __unicode__(self):
         return '{} {} {}'.format(
             self.supply,
@@ -55,6 +70,47 @@ class BuildEvent(TrackerEvent):
         )
 
 
+class DiedEvent(TrackerEvent):
+    def __init__(self, name, frame, killer):
+        self.name = name
+        super(DiedEvent, self).__init__(frame, None)
+        self.killer = killer  # player id
+
+    def to_dict(self):
+        return {
+            'frame': self.frame,
+            'time': _frame_to_time(self.frame),
+            'name': self.name,
+            'killer': self.killer,
+        }
+
+    def __unicode__(self):
+        return '{} {} killed by {}'.format(
+            _frame_to_time(self.frame),
+            self.name,
+            self.killer,
+            )
+
+
+class AbilityEvent(TrackerEvent):
+    def __init__(self, name, frame):
+        self.name = name
+        super(AbilityEvent, self).__init__(frame, None)
+
+    def to_dict(self):
+        return {
+            'frame': self.frame,
+            'time': _frame_to_time(self.frame),
+            'name': self.name,
+        }
+
+    def __unicode__(self):
+        return '{} {}'.format(
+            _frame_to_time(self.frame),
+            self.name,
+            )
+
+
 class GameTimeline(object):
     def __init__(self):
         self.timeline = []
@@ -106,19 +162,27 @@ def get_supply(supply, frame):
 def unit_born_event(builds, event, parsed_data):
     """
     need to reverse the time
+    if the unit morphs, it's preferable to use the unit_type_name since it's what it was
+    actually born as. sc2reader does, however, provide some nice normalization, so
+    swap the display name as necessary
     """
     player = event.control_pid
-    display_name = event.unit.name
     unit_name = event.unit_type_name
     if unit_name in BO_EXCLUDED or player == 0:
         return
+
+    if not unit_name in BUILD_TIMES:
+        unit_name = event.unit.name
+    if unit_name is None:
+        unit_name = '(None)'
+
     try:
         frame = event.frame - BUILD_TIMES[unit_name]
     except KeyError:
         frame = event.frame
-        display_name += ' (Error on time)'
+        unit_name += ' (Error on born time)'
     supply = get_supply(parsed_data['players'][player]['supply'], frame)
-    builds[player].add_event(BuildEvent(display_name, frame, supply))
+    builds[player].add_event(BuildEvent(unit_name, frame, supply))
 
 
 def unit_init_event(builds, event, parsed_data):
@@ -143,20 +207,62 @@ def upgrade_event(builds, event, parsed_data):
         frame = event.frame - BUILD_TIMES[unit_name]
     except KeyError:
         frame = event.frame
-        unit_name += ' (Error on time)'
+        unit_name += ' (Error on upgrade time)'
     supply = get_supply(parsed_data['players'][player]['supply'], frame)
     builds[player].add_event(BuildEvent(unit_name, frame, supply))
 
 
-def make_event_timeline(builds, cutoff_time, parsed_data):
+def change_event(builds, event, parsed_data):
+    player = event.unit.owner.pid
+    if player == 0:
+        return
+    unit_name = event.unit_type_name
+    try:
+        frame = event.frame - BUILD_TIMES[unit_name]
+    except KeyError:
+        return
+
+    supply = get_supply(parsed_data['players'][player]['supply'], frame)
+    builds[player].add_event(BuildEvent(unit_name, frame, supply))
+
+
+def died_event(units_lost, event):
+    """
+    for born events, the name needed to be corrected for evolutions
+    but if it dies, then it should die in its final form
+    """
+    if not event.unit or not event.unit.owner:
+        return
+    player = event.unit.owner.pid
+    unit_name = event.unit.name
+
+    if unit_name in BO_EXCLUDED or player == 0:
+        return
+
+    killer = event.killer_pid
+
+    units_lost[player].add_event(DiedEvent(unit_name, event.frame, killer))
+
+
+def ability_event(abilities, event):
+    player = event.player.pid
+    ability_name = event.ability_name
+
+    if ability_name not in TRACKED_ABILITIES or not player:
+        return
+
+    abilities[player].add_event(AbilityEvent(ability_name, event.frame))
+
+
+def make_event_timeline(timelines, cutoff_time, parsed_data, field):
     """
     Converts the GameTimeline into a readable structure
     """
-    for i, build in builds.iteritems():
-        build.sort()
+    for i, timeline in timelines.iteritems():
+        timeline.sort()
 
-        parsed_data['players'][i]['buildOrder'] = []
-        for event in build.timeline:
+        parsed_data['players'][i][field] = []
+        for event in timeline.timeline:
 
             # If desired, stop parsing for events after a given time
             current_gametime = _frame_to_time(event.frame)
@@ -165,15 +271,7 @@ def make_event_timeline(builds, cutoff_time, parsed_data):
                     convert_gametime_to_float(cutoff_time)):
                 break
 
-            parsed_data['players'][i]['buildOrder'].append(
-                {
-                    'frame': event.frame,
-                    'time': _frame_to_time(event.frame),
-                    'name': event.name,
-                    'supply': event.supply,
-                    'is_worker': event.is_worker()
-                }
-            )
+            parsed_data['players'][i][field].append(event.to_dict())
 
 
 def parse_events(replay, cutoff_time, parsed_data):
@@ -194,12 +292,13 @@ def parse_events(replay, cutoff_time, parsed_data):
         raise ReplayFormatError(('spawningtool currently only supports HotS.'), parsed_data)
 
     builds = dict((key, GameTimeline()) for key in replay.player.iterkeys())
+    units_lost = dict((key, GameTimeline()) for key in replay.player.iterkeys())
+    abilities = dict((key, GameTimeline()) for key in replay.player.iterkeys())
 
     for event in replay.tracker_events:
         if event.frame == 0:
             continue
-
-        if event.name == 'PlayerStatsEvent':
+        if event.name == 'PlayerStatsEvent' and event.pid in parsed_data['players']:
             parsed_data['players'][event.pid]['supply'].append(
                 [event.frame, int(event.food_used)])
         elif event.name == 'UnitBornEvent':
@@ -208,9 +307,25 @@ def parse_events(replay, cutoff_time, parsed_data):
             unit_init_event(builds, event, parsed_data)
         elif event.name == 'UpgradeCompleteEvent':
             upgrade_event(builds, event, parsed_data)
+        elif event.name == 'UnitTypeChangeEvent':
+            change_event(builds, event, parsed_data)
+        elif event.name == 'UnitDiedEvent':
+            died_event(units_lost, event)
+
+    legit_ability_event_types = set([
+        'LocationAbilityEvent',
+        'TargetAbilityEvent',
+        'SelfAbilityEvent',
+        'AbilityEvent',
+        ])
+    for event in replay.game_events:
+        if event.name in legit_ability_event_types:
+            ability_event(abilities, event)
 
     parsed_data['buildOrderExtracted'] = True  # legacy code
-    make_event_timeline(builds, cutoff_time, parsed_data)
+    make_event_timeline(builds, cutoff_time, parsed_data, 'buildOrder')
+    make_event_timeline(units_lost, cutoff_time, parsed_data, 'unitsLost')
+    make_event_timeline(abilities, cutoff_time, parsed_data, 'abilities')
     return parsed_data
 
 
@@ -222,6 +337,8 @@ def parse_replay(replay_file, cutoff_time=None):
         replay = sc2reader.load_replay(replay_file)
     except sc2reader.exceptions.ReadError as error:
         raise ReadError(error.message)
+    except Exception as error:  # Maybe a MPQError
+        raise ReadError(error.message)
 
     parsed_data = {
         'buildOrderExtracted': False,
@@ -249,7 +366,7 @@ def parse_replay(replay_file, cutoff_time=None):
                 'is_human': player.is_human,
                 'handicap': player.handicap,
                 'color': player.color.hex,
-                'uid': player.uid,
+                'uid': player.toon_id,  # naming change from sc2reader
                 'region': player.region,
                 'supply': [[0, 6]],
                 'team': player.team.number,
